c**********************************************************************
c IO_INIT_FILTER.F
c**********************************************************************
c Read in filter parameters, precompute filters.
c
c Filter/decimation process was published in Quinn, T. R., et al.:
c Integration of Earth's orbit. AJ 102, p. 2287, 1991
c
c These subroutines and functions are used to obtain long term
c variations of orbital elements.
c
c Input:
c  infile	name of the filter parameters input file,
c		for structure of this file refer to example filter.in
c  ntp		number of TP's (due to check the buffer size)
c
c Output:
c  dtfilter			sampling period
c  outfile			name of binary output file
c  iflgchk			real*8 and real*4 output flags
c
c  in common block /filter/
c  see filter_cb.inc
c
c Remarks: 
c Author: Miroslav Broz, miroslav.broz@email.cz
c Date: Apr 8th 2003

      subroutine io_init_filter(infile,ntp,dtfilter_param,outfile,
     :  iflgchk)

      include 'swift.inc'
      include 'filter.inc'
      include 'flt_cb.inc'

c  input
      character*80 infile
      integer ntp

c  output
      real*8 dtfilter_param
      character*80 outfile
      integer iflgchk

c  internals
      integer i,ierr,AM,BM
      real*8 Ax0,Bx0,Abeta,Bbeta
      logical write_r8

c  functions
      integer length

c  main ----------------------------------------------------------

c Check number of test particles, filtration subroutines needs to allocate
c buffer for orbital elements elmts(FMAXN,6,FMAXNTP)
c Filter parameters are given to io_write_filter.f in /filter/ common block

      if (ntp.gt.FMAXNTP) then
        write(*,*) 'Number of test particles ntp =',ntp,' exceeds',
     &    ' max number FMAXNTP =',FMAXNTP, '.'
        call util_exit(1)
      endif

c  read input parameters
      write(*,*) 'Filter parameters file called ',infile
      call io_open(7,infile,'old','formatted',ierr)

      read(7,*) filter_N
      read(7,10) filter_stat
10    format(a)
      read(7,*) (filter_D(i),i=1,filter_N)
      read(7,*) AM,Ax0,Abeta
      read(7,*) BM,Bx0,Bbeta
c
c  sampling period, output to outfile will be produced every
c  filter_D(1) * filter_D(2) * ... * filter_D(N) * dtfilter, see FBUFN
c
      read(7,*) dtfilter
      read(7,10) outfile
      read(7,*) write_r8
      read(7,*) filter_write
      read(7,*) filter_write_r8

      close(unit = 7)
      write(*,*) ' '

c  check the number of filters
      if (filter_N.ne.length(filter_stat)) then
        write(*,*) 'Number of filters ',filter_N,' differs',
     &    ' from length of filter sequence ',
     &    filter_stat(1:length(filter_stat)), '.'
        call util_exit(1)
      endif

      call filter_create(A,AM,Ax0,Abeta)
      call filter_create(B,BM,Bx0,Bbeta)
      filter_M = AM		! expecting AM=BM
      FBUFN = 2*filter_M+1	! width of buffers

c  give dtfilter both as a parameter and in common block 
      dtfilter_param = dtfilter

c  setup iflgchk bits, according to options
c  the summary of iflgchk bits follows:
c     6 ... reorientation
c     7 ... disruption
c     8 ... write real*8 bin.dat

      if (write_r8) iflgchk=ibset(iflgchk,8)

c  FNdtout * dtfilter ... output (filtered) timestep
c  (ie. multiplied decimation factors) 
      FNdtout = 1
      do i = 1, filter_N
        FNdtout = FNdtout * filter_D(i)
      enddo

c  FNinibuf ... number of point needed to fill the buffer INITIALLY
c  (we will have to shift output time by FNinibuf * dtfilter / 2)
      FNinibuf = FBUFN
      do i = filter_N-1, 1, -1
        FNinibuf = filter_D(i)*(FNinibuf-1) + FBUFN
      enddo 
      FNinibuf = FNinibuf - 2	! empirically "measured" factor?!

      return
      end    ! io_init_filter
c-----------------------------------------------------------------

c****************************************************************
      double precision function I0(x)
c****************************************************************
c
c  Modified Bessel function
c  from Numerical Recipes, p. 273, function bessi0
c  http://cfatab.harvard.edu/nr/bookf.html
c
      implicit none
      double precision x
      double precision ax,p1,p2,p3,p4,p5,p6,p7,
     &  q1,q2,q3,q4,q5,q6,q7,q8,q9,y
      save p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7,q8,q9
      data p1,p2,p3,p4,p5,p6,p7/1.0d0,3.5156229d0,3.0899424d0,
     &  1.2067492d0,0.2659732d0,0.360768d-1,0.45813d-2/
      data q1,q2,q3,q4,q5,q6,q7,q8,q9/0.39894228d0,0.1328592d-1,
     &  0.225319d-1,-0.157565d-1,0.916281d-2,-0.2057706,
     &  0.2635537d-1,-0.1647633d-1,0.392377d-1/

      if (abs(x).lt.3.75) then
        y=(x/3.75)**2
        I0=p1+y*(p2+y*(p3+y*(p4+y*(p5+y*(p6+y*p7)))))
      else
        ax=abs(x)
        y=3.75/ax
        I0=(exp(ax)/sqrt(ax))*(q1+y*(q2+y*(q3+y*(q4
     &     +y*(q5+y*(q6+y*(q7+y*(q8+y*q9))))))))
      endif
      return
      end

c****************************************************************
      subroutine filter_create(d,Mm,x0,beta)
c****************************************************************
c
c  create filter
c  d(Mm)       filter
c  Mm,x0,beta  parameters of the filter
c
      implicit none
      include 'filter.inc'
      real*8 pi
      parameter (pi = 3.14159265358979d0)

c  parameters
      integer Mm
      double precision x0,beta,d(-FMAXM:FMAXM)
c  local varibles
      integer m
      double precision C,I0

      d(0)=2*x0*I0(beta)
      do m=1,Mm      
        d(m)=sin(2*pi*m*x0)/(pi*m)*I0(beta*sqrt(1-m*m/dble(Mm*Mm)))
        d(-m)=d(m)
      enddo
      C=0
      do m=-Mm,Mm
        C=C+d(m)
      enddo
      C=1/C
      do m=-Mm,Mm
        d(m)=C*d(m)
      enddo
      return
      end

c****************************************************************
       double precision function filter_filter(d,w,Mm)
c****************************************************************
c
c  filter the data w with filter d
c  d    filter
c  Mn   dimension of the filter and data
c  w    input data
c
      implicit none
      include 'filter.inc'
      double precision d(-FMAXM:FMAXM),w(0:FMAXN-1)
      integer Mm

      integer m
      double precision w2

      w2=0.0d0
      do m=-Mm,Mm
        w2=w2+d(m)*w(Mm-m)
      enddo
      filter_filter=w2
      return
      end

